import { GameSettings } from "@components/Lobby/SettingsFrame/Setting"
import { GamePropsSchema } from "@lib/zodSchemas"
import { produce } from "immer"
import { create } from "zustand"
import { devtools } from "zustand/middleware"

const initialState: GamePropsSchema = { payload: null, hash: null }

type State = GamePropsSchema

type Action = {
  setSetting: (by: GameSettings) => void
  full: (newProps: GamePropsSchema) => void
  reset: () => void
}

export const useGameProps = create<State & Action>()(
  devtools(
    (set) => ({
      ...initialState,
      setSetting: (settings) =>
        set(
          produce((state: State) => {
            if (!state.payload?.game) return
            Object.assign(state.payload.game, settings)
          })
        ),
      full: (newGameProps) =>
        set((state) => {
          if (state.hash === newGameProps.hash) {
            console.log("Everything up to date.")
          } else {
            console.log("Update was needed.", state.hash, newGameProps.hash)

            if (
              state.payload?.game?.id &&
              state.payload?.game?.id !== newGameProps.payload?.game?.id
            ) {
              console.warn(
                "Different gameId detected on update: ",
                state.payload?.game?.id,
                newGameProps.payload?.game?.id
              )
            }

            return newGameProps
          }
          return state
        }),
      reset: () => {
        set(initialState)
      },
    }),
    {
      name: "gameState",
    }
  )
)
