import { useGameProps } from "@hooks/useGameProps"
import { socket } from "@lib/socket"
import { useEffect, useState } from "react"
import { toast } from "react-toastify"

/** This function should only be called once per page, otherwise there will be multiple socket connections and duplicate event listeners. */
function useSocket() {
  const [isConnected, setIsConnected] = useState(false)
  const { full } = useGameProps()

  useEffect(() => setIsConnected(socket.connected), [socket.connected])

  useEffect(() => {
    socket.connect()

    socket.on("connect", () => {
      console.log("connected")

      const toastId = "connect_error"
      toast.dismiss(toastId)
    })

    socket.on("connect_error", () => {
      const toastId = "connect_error"
      const isActive = toast.isActive(toastId)
      if (isActive)
        toast.update(toastId, {
          autoClose: 5000,
        })
      else
        toast.warn("Es gibt Probleme mit der Echtzeitverbindung.", { toastId })
    })

    socket.on("disconnect", () => {
      console.log("disconnect")
    })

    socket.on("update", (body) => {
      console.log("update")
      full(body)
    })

    return () => {
      socket.removeAllListeners()
      socket.disconnect()
    }
  }, [])

  // useEffect(() => {
  //   const toastId = "realtime"

  //   toast("Echtzeitverbindung wird hergestellt...", {
  //     icon: Icons.spinner(),
  //     toastId,
  //     autoClose: false,
  //     hideProgressBar: true,
  //     closeButton: false,
  //   })
  //   socket.emit("join", ({ ack }) => {
  //     if (!ack) {
  //       toast.update(toastId, {
  //         render: "Bei der Echtzeitverbindung ist ein Fehler aufgetreten 🤯",
  //         type: "error",
  //         icon: Icons.error,
  //         theme: "colored",
  //         autoClose: 5000,
  //         hideProgressBar: false,
  //         closeButton: true,
  //       })
  //     } else {
  //       setHasJoined(true)
  //       toast.update(toastId, {
  //         render: "Echtzeitverbindung hergestellt 👌",
  //         type: "info",
  //         icon: Icons.success,
  //         autoClose: 5000,
  //         hideProgressBar: false,
  //         closeButton: true,
  //       })
  //     }
  //   })
  // }, [])

  // useEffect(() => {
  //   if (!isConnected) return
  //   socket.emit("authenticate", { token: `hello from ${session?.user.email}` })
  // }, [isConnected, status, session?.user.email])

  // useEffect(() => {
  //   if (!isConnected) return
  //   let count = 0
  //   const interval = setInterval(() => {
  //     const start = Date.now()
  //     socket.volatile.emit("ping", ++count, (count) => {
  //       const duration = Date.now() - start
  //       console.log("ping", count, duration)
  //     })
  //   }, 5000)
  //   return () => clearInterval(interval)
  // }, [isConnected])

  return { isConnected }
}

export default useSocket
